function [models, timeLag, dimension] = learnModels(timeSeries, M)

% [models, timeLag, dimension] = learnModels(timeSeries, M)
%
% Input Variables
%       timeSeries - a set of signals and their labels
%       M - number of mixtures for each class
%
% Output Variables
%       models - the models for each class.
%       timeLag - time lag to use for the reconstructed phase spaces
%       dimension - dimension to use for the reconstructed phase spaces
%
% Language - MATLAB
%  
% Description:  Builds GMM models over RPS for each class
%
% Created:     
%               Date: 6/20/2003  
%               By:  Richard J. Povinelli
%               Marquette University
%
% Modified:    
%               Version:
%               Date: 
%               By: 
%               Why: 

[numberOfSignals dummy] = size(timeSeries);
% normalize each time series to zero mean and unit variance
for i = 1:numberOfSignals
  signal = timeSeries{i,2};
  signalMean = mean(signal);
  signalStd = std(signal);
  timeSeries{i,2} = (signal - signalMean)/signalStd;
end %for

timeLag = determineTimeLag(timeSeries(:,2));
dimension = determineDimension(timeSeries(:,2), timeLag);
lags = createLags(dimension,timeLag);

allLabels = timeSeries(:,1); %copies the labels out of data
uniqueLabels = unique(allLabels); %find the unique class labels
numberOfClasses = length(uniqueLabels); %find the number of classes

% set up the options for GMM
covarianceType = 'full';
options(1) = -1;  % no error display
options(5) = 1;   % avoid covariance collapse
options(14) = 25; % number of EM iterations
options(19) = 1;  % floor on covariance matrix
models = cell(numberOfClasses,2);

for i = 1:numberOfClasses %form a reconstructed phase space and insert corresponding signals
  rps = []; %the embedding with all the signals
  classIndices = strmatch(uniqueLabels(i),allLabels,'exact'); %get indices for each class
  elementsInClass = length(classIndices);
  for j = 1:elementsInClass %build a RPS for the class
    rps = [rps embed(timeSeries{classIndices(j),2},lags)];
  end %for
%   uniqueLabels(i) %debug
  models(i,1) = uniqueLabels(i);
  models(i,2) = {gmmemTrain(covarianceType,M,rps',options)}; %using EM learn GMM
end %for

return